# -*- coding: utf-8 -*-
# Copyright (C) 2011-2016 Avencall
# Copyright (C) 2016 Proformatique Inc.
# SPDX-License-Identifier: GPL-3.0+

import logging
import time

from xivo_confgen import cache
from xivo_confgen.asterisk import AsteriskFrontend
from xivo_confgen.xivo import XivoFrontend
from xivo_confgen.dird import DirdFrontend
from xivo_confgen.dird_phoned import DirdPhonedFrontend
from xivo_confgen.handler import CachedHandlerFactoryDecorator
from xivo_confgen.handler import MultiHandlerFactory
from xivo_confgen.handler import PluginHandlerFactory
from xivo_confgen.handler import FrontendHandlerFactory
from xivo_confgen.handler import NullHandlerFactory
from xivo_confgen.template import new_template_helper
from xivo_dao.helpers.db_utils import session_scope
from twisted.internet.protocol import Protocol, ServerFactory

logger = logging.getLogger(__name__)


class Confgen(Protocol):

    def dataReceived(self, data):
        try:
            t1 = time.time()
            try:
                resource, filename = data.replace('\n', '').split('/')
            except ValueError:
                logger.error("cannot split %s", data)
                return

            content = self.factory.generate(resource, filename)
            if content:
                self.transport.write(content)
            t2 = time.time()

            logger.info("serving %s in %.3f seconds", data, t2 - t1)
        finally:
            self.transport.loseConnection()


class ConfgendFactory(ServerFactory):

    protocol = Confgen

    def __init__(self, cachedir, config):
        tpl_helper = new_template_helper()
        dependencies = {
            'config': config,
            'tpl_helper': tpl_helper,
        }
        frontends = {
            'asterisk': AsteriskFrontend(config, tpl_helper),
            'dird': DirdFrontend(),
            'dird-phoned': DirdPhonedFrontend(),
            'xivo': XivoFrontend(),
        }
        self._cache = cache.FileCache(cachedir)
        self._handler_factory = MultiHandlerFactory([CachedHandlerFactoryDecorator(PluginHandlerFactory(config, dependencies)),
                                                     FrontendHandlerFactory(frontends),
                                                     NullHandlerFactory()])

    def generate(self, resource, filename):
        cache_key = '{}/{}'.format(resource, filename)
        handler = self._handler_factory.get(resource, filename)
        with session_scope():
            try:
                content = handler()
            except Exception:
                logger.error('unexpected error raised by handler', exc_info=True)
                content = self._get_cached_content(cache_key)
        return self._encode_and_cache(cache_key, content)

    def _get_cached_content(self, cache_key):
        logger.info("cache hit on %s", cache_key)
        try:
            return self._cache.get(cache_key).decode('utf-8')
        except AttributeError:
            logger.warning("No cached content for %s", cache_key)

    def _encode_and_cache(self, cache_key, content):
        if not content:
            return

        encoded_content = content.encode('utf-8')
        self._cache.put(cache_key, encoded_content)
        return encoded_content
